@@ -26,7 +26,7 @@
 #ifndef _COMPIZ_H
 #define _COMPIZ_H
 
-#define ABIVERSION 20061112
+#define ABIVERSION 20061227
 
 #include <stdio.h>
 #include <sys/time.h>
@@ -46,6 +46,10 @@
 #include <GL/gl.h>
 #include <GL/glx.h>
 
+#ifdef  __cplusplus
+extern "C" {
+#endif
+
 #if COMPOSITE_MAJOR > 0 || COMPOSITE_MINOR > 2
 #define USE_COW
 #endif
@@ -69,12 +73,13 @@
 #define  _(x) x
 #define N_(x) x
 
-typedef struct _CompPlugin  CompPlugin;
-typedef struct _CompDisplay CompDisplay;
-typedef struct _CompScreen  CompScreen;
-typedef struct _CompWindow  CompWindow;
-typedef struct _CompTexture CompTexture;
-typedef struct _CompIcon    CompIcon;
+typedef struct _CompPlugin	  CompPlugin;
+typedef struct _CompDisplay	  CompDisplay;
+typedef struct _CompScreen	  CompScreen;
+typedef struct _CompWindow	  CompWindow;
+typedef struct _CompTexture	  CompTexture;
+typedef struct _CompIcon	  CompIcon;
+typedef struct _CompWindowExtents CompWindowExtents;
 
 /* virtual modifiers */
 
@@ -253,33 +258,6 @@ freePrivateIndex (int  len,
 		  int  index);
 
 
-/* readpng.c */
-
-Bool
-openImageFile (const char *filename,
-	       char	  **returnFilename,
-	       FILE	  **returnFile);
-
-Bool
-readPng (const char   *filename,
-	 char	      **data,
-	 unsigned int *width,
-	 unsigned int *height);
-
-Bool
-readPngBuffer (const unsigned char *buffer,
-	       char		   **data,
-	       unsigned int	   *width,
-	       unsigned int	   *height);
-
-Bool
-writePngToFile (unsigned char *buffer,
-		char	      *filename,
-		int	      width,
-		int	      height,
-		int	      stride);
-
-
 /* option.c */
 
 typedef enum {
@@ -495,11 +473,15 @@ stringToColor (const char     *color,
 char *
 colorToString (unsigned short *rgba);
 
+char *
+optionTypeToString (CompOptionType type);
+
 
 /* display.c */
 
 typedef int CompTimeoutHandle;
 typedef int CompWatchFdHandle;
+typedef int CompFileWatchHandle;
 
 #define COMP_DISPLAY_OPTION_ACTIVE_PLUGINS                0
 #define COMP_DISPLAY_OPTION_TEXTURE_FILTER                1
@@ -589,6 +571,45 @@ typedef Bool (*CallBackProc) (void *closure);
 typedef void (*ForEachWindowProc) (CompWindow *window,
 				   void	      *closure);
 
+typedef Bool (*FileToImageProc) (CompDisplay *display,
+				 const char  *path,
+				 const char  *name,
+				 int	     *width,
+				 int	     *height,
+				 int	     *stride,
+				 void	     **data);
+
+typedef Bool (*ImageToFileProc) (CompDisplay *display,
+				 const char  *path,
+				 const char  *name,
+				 const char  *format,
+				 int	     width,
+				 int	     height,
+				 int	     stride,
+				 void	     *data);
+
+#define NOTIFY_CREATE_MASK (1 << 0)
+#define NOTIFY_DELETE_MASK (1 << 1)
+#define NOTIFY_MOVE_MASK   (1 << 2)
+
+typedef void (*FileWatchCallBackProc) (const char *name,
+				       void	  *closure);
+
+typedef struct _CompFileWatch {
+    struct _CompFileWatch *next;
+    char		  *path;
+    int			  mask;
+    FileWatchCallBackProc callBack;
+    void		  *closure;
+    CompFileWatchHandle   handle;
+} CompFileWatch;
+
+typedef void (*FileWatchAddedProc) (CompDisplay	  *display,
+				    CompFileWatch *fileWatch);
+
+typedef void (*FileWatchRemovedProc) (CompDisplay   *display,
+				      CompFileWatch *fileWatch);
+
 struct _CompDisplay {
     Display    *display;
     CompScreen *screens;
@@ -725,6 +746,8 @@ struct _CompDisplay {
     Atom xdndEnterAtom;
     Atom xdndLeaveAtom;
     Atom xdndPositionAtom;
+    Atom xdndStatusAtom;
+    Atom xdndDropAtom;
 
     Atom managerAtom;
     Atom targetsAtom;
@@ -760,6 +783,8 @@ struct _CompDisplay {
     CompOptionValue plugin;
     Bool	    dirtyPluginList;
 
+    CompFileWatch *fileWatch;
+
     SetDisplayOptionProc	  setDisplayOption;
     SetDisplayOptionForPluginProc setDisplayOptionForPlugin;
 
@@ -769,6 +794,12 @@ struct _CompDisplay {
     HandleEventProc       handleEvent;
     HandleCompizEventProc handleCompizEvent;
 
+    FileToImageProc fileToImage;
+    ImageToFileProc imageToFile;
+
+    FileWatchAddedProc   fileWatchAdded;
+    FileWatchRemovedProc fileWatchRemoved;
+
     CompPrivate *privates;
 };
 
@@ -801,6 +832,25 @@ compAddWatchFd (int	     fd,
 void
 compRemoveWatchFd (CompWatchFdHandle handle);
 
+CompFileWatchHandle
+addFileWatch (CompDisplay	    *display,
+	      const char	    *path,
+	      int		    mask,
+	      FileWatchCallBackProc callBack,
+	      void		    *closure);
+
+void
+removeFileWatch (CompDisplay	     *display,
+		 CompFileWatchHandle handle);
+
+void
+fileWatchAdded (CompDisplay   *display,
+		CompFileWatch *fileWatch);
+
+void
+fileWatchRemoved (CompDisplay   *display,
+		  CompFileWatch *fileWatch);
+
 int
 compCheckForError (Display *dpy);
 
@@ -868,6 +918,41 @@ setDisplayAction (CompDisplay     *display,
 		  CompOption      *o,
 		  CompOptionValue *value);
 
+Bool
+readImageFromFile (CompDisplay *display,
+		   const char  *name,
+		   int	       *width,
+		   int	       *height,
+		   void	       **data);
+
+Bool
+writeImageToFile (CompDisplay *display,
+		  const char  *path,
+		  const char  *name,
+		  const char  *format,
+		  int	      width,
+		  int	      height,
+		  void	      *data);
+
+Bool
+fileToImage (CompDisplay *display,
+	     const char	 *path,
+	     const char	 *name,
+	     int	 *width,
+	     int	 *height,
+	     int	 *stride,
+	     void	 **data);
+
+Bool
+imageToFile (CompDisplay *display,
+	     const char	 *path,
+	     const char	 *name,
+	     const char	 *format,
+	     int	 width,
+	     int	 height,
+	     int	 stride,
+	     void	 *data);
+
 
 /* event.c */
 
@@ -923,6 +1008,8 @@ typedef struct _WindowPaintAttrib {
     GLushort saturation;
     GLfloat  xScale;
     GLfloat  yScale;
+    GLfloat  xTranslate;
+    GLfloat  yTranslate;
 } WindowPaintAttrib;
 
 extern ScreenPaintAttrib defaultScreenPaintAttrib;
@@ -975,12 +1062,18 @@ typedef void (*ApplyScreenTransformProc) (CompScreen		  *screen,
 #define PAINT_WINDOW_TRANSLUCENT_MASK		(1 << 1)
 #define PAINT_WINDOW_TRANSFORMED_MASK           (1 << 2)
 #define PAINT_WINDOW_ON_TRANSFORMED_SCREEN_MASK (1 << 3)
+#define PAINT_WINDOW_NO_CORE_INSTANCE_MASK	(1 << 4)
 
 typedef Bool (*PaintWindowProc) (CompWindow		 *window,
 				 const WindowPaintAttrib *attrib,
 				 Region			 region,
 				 unsigned int		 mask);
 
+typedef Bool (*DrawWindowProc) (CompWindow		*window,
+				const WindowPaintAttrib *attrib,
+				Region			region,
+				unsigned int		mask);
+
 typedef void (*AddWindowGeometryProc) (CompWindow *window,
 				       CompMatrix *matrix,
 				       int	  nMatrix,
@@ -1060,6 +1153,12 @@ drawWindowTexture (CompWindow		   *w,
 		   const WindowPaintAttrib *attrib,
 		   unsigned int		   mask);
 
+Bool
+drawWindow (CompWindow		    *w,
+	    const WindowPaintAttrib *attrib,
+	    Region		    region,
+	    unsigned int	    mask);
+
 Bool
 paintWindow (CompWindow		     *w,
 	     const WindowPaintAttrib *attrib,
@@ -1116,13 +1215,6 @@ readImageToTexture (CompScreen   *screen,
 		    unsigned int *width,
 		    unsigned int *height);
 
-Bool
-readImageBufferToTexture (CompScreen	      *screen,
-			  CompTexture	      *texture,
-			  const unsigned char *imageBuffer,
-			  unsigned int	      *returnWidth,
-			  unsigned int	      *returnHeight);
-
 Bool
 iconToTexture (CompScreen *screen,
 	       CompIcon   *icon);
@@ -1298,9 +1390,8 @@ typedef Bool (*DamageWindowRectProc) (CompWindow *w,
 typedef Bool (*DamageWindowRegionProc) (CompWindow *w,
 					Region     region);
 
-typedef void (*SetWindowScaleProc) (CompWindow *w,
-				    float      xScale,
-				    float      yScale);
+typedef void (*GetOutputExtentsForWindowProc) (CompWindow	 *w,
+					       CompWindowExtents *output);
 
 typedef Bool (*FocusWindowProc) (CompWindow *window);
 
@@ -1445,6 +1536,7 @@ struct _CompScreen {
     int		      sizeExpose;
     int		      nExpose;
     CompTexture       backgroundTexture;
+    Bool	      backgroundLoaded;
     unsigned int      pendingDestroys;
     int		      desktopWindowCount;
     unsigned int      mapNum;
@@ -1555,19 +1647,20 @@ struct _CompScreen {
     InitPluginForScreenProc initPluginForScreen;
     FiniPluginForScreenProc finiPluginForScreen;
 
-    PreparePaintScreenProc     preparePaintScreen;
-    DonePaintScreenProc	       donePaintScreen;
-    PaintScreenProc	       paintScreen;
-    PaintTransformedScreenProc paintTransformedScreen;
-    ApplyScreenTransformProc   applyScreenTransform;
-    PaintBackgroundProc        paintBackground;
-    PaintWindowProc	       paintWindow;
-    AddWindowGeometryProc      addWindowGeometry;
-    DrawWindowGeometryProc     drawWindowGeometry;
-    DrawWindowTextureProc      drawWindowTexture;
-    DamageWindowRectProc       damageWindowRect;
-    FocusWindowProc	       focusWindow;
-    SetWindowScaleProc	       setWindowScale;
+    PreparePaintScreenProc	  preparePaintScreen;
+    DonePaintScreenProc		  donePaintScreen;
+    PaintScreenProc		  paintScreen;
+    PaintTransformedScreenProc	  paintTransformedScreen;
+    ApplyScreenTransformProc	  applyScreenTransform;
+    PaintBackgroundProc		  paintBackground;
+    PaintWindowProc		  paintWindow;
+    DrawWindowProc		  drawWindow;
+    AddWindowGeometryProc	  addWindowGeometry;
+    DrawWindowGeometryProc	  drawWindowGeometry;
+    DrawWindowTextureProc	  drawWindowTexture;
+    DamageWindowRectProc	  damageWindowRect;
+    GetOutputExtentsForWindowProc getOutputExtentsForWindow;
+    FocusWindowProc		  focusWindow;
 
     WindowResizeNotifyProc windowResizeNotify;
     WindowMoveNotifyProc   windowMoveNotify;
@@ -1607,6 +1700,12 @@ updateScreenBackground (CompScreen  *screen,
 void
 detectRefreshRateOfScreen (CompScreen *s);
 
+void
+showOutputWindow (CompScreen *s);
+
+void
+hideOutputWindow (CompScreen *s);
+
 Bool
 addScreen (CompDisplay *display,
 	   int	       screenNum,
@@ -1796,6 +1895,9 @@ clearScreenOutput (CompScreen   *s,
 		   int	        output,
 		   unsigned int mask);
 
+Bool
+updateDefaultIcon (CompScreen *screen);
+
 
 /* window.c */
 
@@ -1812,12 +1914,12 @@ typedef Bool (*InitPluginForWindowProc) (CompPlugin *plugin,
 typedef void (*FiniPluginForWindowProc) (CompPlugin *plugin,
 					 CompWindow *window);
 
-typedef struct _CompWindowExtents {
+struct _CompWindowExtents {
     int left;
     int right;
     int top;
     int bottom;
-} CompWindowExtents;
+};
 
 typedef struct _CompStruts {
     XRectangle left;
@@ -1899,7 +2001,6 @@ struct _CompWindow {
 
     WindowPaintAttrib paint;
     WindowPaintAttrib lastPaint;
-    Bool	      scaled;
 
     CompWindowExtents input;
     CompWindowExtents output;
@@ -2041,8 +2142,10 @@ setWmState (CompDisplay *display,
 
 void
 setWindowFrameExtents (CompWindow	 *w,
-		       CompWindowExtents *input,
-		       CompWindowExtents *output);
+		       CompWindowExtents *input);
+
+void
+updateWindowOutputExtents (CompWindow *w);
 
 void
 updateWindowRegion (CompWindow *w);
@@ -2116,6 +2219,14 @@ void
 circulateWindow (CompWindow	 *w,
 		 XCirculateEvent *ce);
 
+void
+addWindowDamageRect (CompWindow *w,
+		     BoxPtr     rect);
+
+void
+getOutputExtentsForWindow (CompWindow	     *w,
+			   CompWindowExtents *output);
+
 void
 addWindowDamage (CompWindow *w);
 
@@ -2128,13 +2239,12 @@ damageWindowRect (CompWindow *w,
 		  BoxPtr     rect);
 
 void
-damageWindowRegion (CompWindow *w,
-		    Region     region);
-
-void
-setWindowScale (CompWindow *w,
-		float      xScale,
-		float      yScale);
+damageTransformedWindowRect (CompWindow *w,
+			     float	xScale,
+			     float	yScale,
+			     float	xTranslate,
+			     float	yTranslate,
+			     BoxPtr     rect);
 
 Bool
 focusWindow (CompWindow *w);
@@ -2262,6 +2372,8 @@ setDesktopForWindow (CompWindow   *w,
 
 /* plugin.c */
 
+#define HOME_PLUGINDIR ".compiz/plugins"
+
 typedef int (*GetVersionProc) (CompPlugin *plugin,
 			       int	  version);
 
@@ -2322,8 +2434,12 @@ typedef Bool (*LoadPluginProc) (CompPlugin *p,
 
 typedef void (*UnloadPluginProc) (CompPlugin *p);
 
+typedef char **(*ListPluginsProc) (char *path,
+				   int	*n);
+
 extern LoadPluginProc   loaderLoadPlugin;
 extern UnloadPluginProc loaderUnloadPlugin;
+extern ListPluginsProc  loaderListPlugins;
 
 struct _CompPlugin {
     CompPlugin       *next;
@@ -2384,6 +2500,9 @@ popPlugin (void);
 CompPlugin *
 getPlugins (void);
 
+char **
+availablePlugins (int *n);
+
 
 /* session.c */
 
@@ -2393,4 +2512,8 @@ initSession (char *smPrevClientId);
 void
 closeSession (void);
 
+#ifdef  __cplusplus
+}
+#endif
+
 #endif
