@@ -29,6 +29,7 @@
 #include <math.h>
 #include <sys/time.h>
 
+#include <X11/Xatom.h>
 #include <X11/cursorfont.h>
 
 #include <compiz.h>
@@ -98,6 +99,10 @@ static IconOverlay iconOverlay[] = {
 #define N_ICON_TYPE (sizeof (iconOverlayString) / sizeof (iconOverlayString[0]))
 #define SCALE_ICON_DEFAULT (iconOverlayString[1])
 
+#define SCALE_HOVER_TIME_DEFAULT 750
+#define SCALE_HOVER_TIME_MIN     50
+#define SCALE_HOVER_TIME_MAX     10000
+
 static int displayPrivateIndex;
 
 typedef struct _ScaleSlot {
@@ -106,9 +111,11 @@ typedef struct _ScaleSlot {
     float scale;
 } ScaleSlot;
 
-#define SCALE_DISPLAY_OPTION_INITIATE     0
-#define SCALE_DISPLAY_OPTION_INITIATE_ALL 1
-#define SCALE_DISPLAY_OPTION_NUM          2
+#define SCALE_DISPLAY_OPTION_INITIATE        0
+#define SCALE_DISPLAY_OPTION_INITIATE_ALL    1
+#define SCALE_DISPLAY_OPTION_INITIATE_GROUP  2
+#define SCALE_DISPLAY_OPTION_INITIATE_OUTPUT 3
+#define SCALE_DISPLAY_OPTION_NUM             4
 
 typedef struct _ScaleDisplay {
     int		    screenPrivateIndex;
@@ -129,7 +136,15 @@ typedef struct _ScaleDisplay {
 #define SCALE_SCREEN_OPTION_DARKEN_BACK  5
 #define SCALE_SCREEN_OPTION_OPACITY      6
 #define SCALE_SCREEN_OPTION_ICON         7
-#define SCALE_SCREEN_OPTION_NUM          8
+#define SCALE_SCREEN_OPTION_HOVER_TIME   8
+#define SCALE_SCREEN_OPTION_NUM          9
+
+typedef enum {
+    ScaleTypeNormal = 0,
+    ScaleTypeOutput,
+    ScaleTypeGroup,
+    ScaleTypeAll
+} ScaleType;
 
 typedef struct _ScaleScreen {
     int windowPrivateIndex;
@@ -139,7 +154,6 @@ typedef struct _ScaleScreen {
     PaintScreenProc        paintScreen;
     PaintWindowProc        paintWindow;
     DamageWindowRectProc   damageWindowRect;
-    WindowMoveNotifyProc   windowMoveNotify;
 
     CompOption opt[SCALE_SCREEN_OPTION_NUM];
 
@@ -150,7 +164,12 @@ typedef struct _ScaleScreen {
 
     unsigned int wMask;
 
-    int grabIndex;
+    Bool grab;
+    int  grabIndex;
+
+    Window dndTarget;
+
+    CompTimeoutHandle hoverHandle;
 
     int state;
     int moreAdjust;
@@ -171,8 +190,9 @@ typedef struct _ScaleScreen {
 
     IconOverlay iconOverlay;
 
-    Bool allWindows;
-    Bool scaleMove;
+    ScaleType type;
+
+    Window clientLeader;
 } ScaleScreen;
 
 typedef struct _ScaleWindow {
@@ -187,8 +207,7 @@ typedef struct _ScaleWindow {
     float   delta;
     Bool    adjust;
 
-    int restoreX;
-    int restoreY;
+    float lastThumbOpacity;
 } ScaleWindow;
 
 
@@ -301,6 +320,9 @@ scaleSetScreenOption (CompScreen      *screen,
 	    }
 	}
 	break;
+    case SCALE_SCREEN_OPTION_HOVER_TIME:
+	if (compSetIntOption (o, value))
+	    return TRUE;
     default:
 	break;
     }
@@ -389,6 +411,28 @@ scaleScreenInitOptions (ScaleScreen *ss)
     o->value.s	      = strdup (SCALE_ICON_DEFAULT);
     o->rest.s.string  = iconOverlayString;
     o->rest.s.nString = N_ICON_TYPE;
+
+    o = &ss->opt[SCALE_SCREEN_OPTION_HOVER_TIME];
+    o->name	  = "hover_time";
+    o->shortDesc  = N_("Hover Time");
+    o->longDesc	  = N_("Time (in ms) before scale mode is terminated when "
+		       "hovering over a window");
+    o->type	  = CompOptionTypeInt;
+    o->value.i    = SCALE_HOVER_TIME_DEFAULT;
+    o->rest.i.min = SCALE_HOVER_TIME_MIN;
+    o->rest.i.max = SCALE_HOVER_TIME_MAX;
+}
+
+static Bool
+isNeverScaleWin (CompWindow *w)
+{
+    if (w->attrib.override_redirect)
+	return TRUE;
+
+    if (w->wmType & (CompWindowTypeDockMask | CompWindowTypeDesktopMask))
+	return TRUE;
+
+    return FALSE;
 }
 
 static Bool
@@ -396,21 +440,18 @@ isScaleWin (CompWindow *w)
 {
     SCALE_SCREEN (w->screen);
 
-    if (!ss->allWindows)
+    if (isNeverScaleWin (w))
+	return FALSE;
+
+    if (!ss->type)
     {
 	if (!(*w->screen->focusWindow) (w))
 	    return FALSE;
     }
 
-    if (w->attrib.override_redirect)
-	return FALSE;
-
     if (!(ss->wMask & w->type))
 	return FALSE;
 
-    if (w->wmType & (CompWindowTypeDockMask | CompWindowTypeDesktopMask))
-	return FALSE;
-
     if (w->state & CompWindowStateSkipPagerMask)
 	return FALSE;
 
@@ -420,6 +461,19 @@ isScaleWin (CompWindow *w)
     if (!w->mapNum || w->attrib.map_state != IsViewable)
 	return FALSE;
 
+    switch (ss->type) {
+    case ScaleTypeGroup:
+	if (ss->clientLeader != w->clientLeader &&
+	    ss->clientLeader != w->id)
+	    return FALSE;
+	break;
+    case ScaleTypeOutput:
+	if (outputDeviceForWindow(w) != w->screen->currentOutputDev)
+	    return FALSE;
+    default:
+	break;
+    }
+
     return TRUE;
 }
 
@@ -437,13 +491,12 @@ scalePaintWindow (CompWindow		  *w,
     if (ss->state != SCALE_STATE_NONE)
     {
 	WindowPaintAttrib sAttrib = *attrib;
+	Bool		  scaled = FALSE;
 
 	SCALE_WINDOW (w);
 
 	if (sw->adjust || sw->slot)
 	{
-	    mask |= PAINT_WINDOW_TRANSFORMED_MASK;
-
 	    if (w->id	    != s->display->activeWindow &&
 		ss->opacity != OPAQUE			&&
 		ss->state   != SCALE_STATE_IN)
@@ -451,18 +504,45 @@ scalePaintWindow (CompWindow		  *w,
 		/* modify opacity of windows that are not active */
 		sAttrib.opacity = (sAttrib.opacity * ss->opacity) >> 16;
 	    }
+
+	    scaled = TRUE;
+
+	    mask |= PAINT_WINDOW_NO_CORE_INSTANCE_MASK;
 	}
-	else if (ss->darkenBack && ss->state != SCALE_STATE_IN)
+	else if (ss->state != SCALE_STATE_IN)
 	{
-	    /* modify brightness of the other windows */
-	    sAttrib.brightness = sAttrib.brightness / 2;
+	    if (ss->darkenBack)
+	    {
+		/* modify brightness of the other windows */
+		sAttrib.brightness = sAttrib.brightness / 2;
+	    }
+
+	    /* hide windows on this output that are not in scale mode */
+	    if (!isNeverScaleWin (w) &&
+		outputDeviceForWindow (w) == s->currentOutputDev)
+	    {
+		sAttrib.opacity = 0;
+	    }
 	}
 
 	UNWRAP (ss, s, paintWindow);
 	status = (*s->paintWindow) (w, &sAttrib, region, mask);
 	WRAP (ss, s, paintWindow, scalePaintWindow);
 
-	if ((ss->iconOverlay != ScaleIconNone) && (sw->adjust || sw->slot))
+	if (scaled)
+	{
+	    WindowPaintAttrib sa = w->lastPaint;
+
+	    sa.xScale     = sw->scale;
+	    sa.yScale     = sw->scale;
+	    sa.xTranslate = sw->tx;
+	    sa.yTranslate = sw->ty;
+
+	    (*s->drawWindow) (w, &sa, region,
+			      mask | PAINT_WINDOW_TRANSFORMED_MASK);
+	}
+
+	if ((ss->iconOverlay != ScaleIconNone) && scaled)
 	{
 	    CompIcon *icon;
 
@@ -515,16 +595,21 @@ scalePaintWindow (CompWindow		  *w,
 		    break;
 		}
 
+		wx += sw->tx;
+		wy += sw->ty;
+
 		if (sw->slot)
 		{
 		    sw->delta =
-			fabs (sw->slot->x1 - sw->restoreX) +
-			fabs (sw->slot->y1 - sw->restoreY) +
+			fabs (sw->slot->x1 - w->attrib.x) +
+			fabs (sw->slot->y1 - w->attrib.y) +
 			fabs (1.0f - sw->slot->scale) * 500.0f;
 		}
 
 		if (sw->delta)
 		{
+		    float o;
+
 		    ds =
 			fabs (sw->tx) +
 			fabs (sw->ty) +
@@ -533,7 +618,22 @@ scalePaintWindow (CompWindow		  *w,
 		    if (ds > sw->delta)
 			ds = sw->delta;
 
-		    sAttrib.opacity = (ds * sAttrib.opacity) / sw->delta;
+		    o = ds / sw->delta;
+
+		    if (sw->slot)
+		    {
+			if (o < sw->lastThumbOpacity)
+			    o = sw->lastThumbOpacity;
+		    }
+		    else
+		    {
+			if (o > sw->lastThumbOpacity)
+			    o = 0.0f;
+		    }
+
+		    sw->lastThumbOpacity = o;
+
+		    sAttrib.opacity = sAttrib.opacity * o;
 		}
 
 		mask |= PAINT_WINDOW_TRANSLUCENT_MASK;
@@ -731,12 +831,9 @@ fillInWindows (CompScreen *s)
 
 	    sw->slot->filled = TRUE;
 
-	    if (!sw->adjust)
-	    {
-		sw->restoreX = w->serverX;
-		sw->restoreY = w->serverY;
-		sw->adjust   = TRUE;
-	    }
+	    sw->lastThumbOpacity = 0.0f;
+
+	    sw->adjust = TRUE;
 	}
     }
 
@@ -823,12 +920,12 @@ adjustScaleVelocity (CompWindow *w)
     }
     else
     {
-	x1 = sw->restoreX;
-	y1 = sw->restoreY;
+	x1 = w->attrib.x;
+	y1 = w->attrib.y;
 	scale = 1.0f;
     }
 
-    dx = x1 - (sw->restoreX + sw->tx);
+    dx = x1 - (w->attrib.x + sw->tx);
 
     adjust = dx * 0.15f;
     amount = fabs (dx) * 1.5f;
@@ -839,7 +936,7 @@ adjustScaleVelocity (CompWindow *w)
 
     sw->xVelocity = (amount * sw->xVelocity + adjust) / (amount + 1.0f);
 
-    dy = y1 - (sw->restoreY + sw->ty);
+    dy = y1 - (w->attrib.y + sw->ty);
 
     adjust = dy * 0.15f;
     amount = fabs (dy) * 1.5f;
@@ -867,8 +964,8 @@ adjustScaleVelocity (CompWindow *w)
 	fabs (ds) < 0.001f && fabs (sw->scaleVelocity) < 0.002f)
     {
 	sw->xVelocity = sw->yVelocity = sw->scaleVelocity = 0.0f;
-	sw->tx = x1 - sw->restoreX;
-	sw->ty = y1 - sw->restoreY;
+	sw->tx = x1 - w->attrib.x;
+	sw->ty = y1 - w->attrib.y;
 	sw->scale = scale;
 
 	return 0;
@@ -898,27 +995,6 @@ scalePaintScreen (CompScreen		  *s,
     return status;
 }
 
-static void
-scaleWindowMoveNotify (CompWindow *w,
-		       int	  dx,
-		       int	  dy,
-		       Bool	  immediate)
-{
-    SCALE_SCREEN (w->screen);
-
-    if (!ss->scaleMove && ss->state != SCALE_STATE_NONE)
-    {
-	SCALE_WINDOW (w);
-
-	sw->restoreX += dx;
-	sw->restoreY += dy;
-    }
-
-    UNWRAP (ss, w->screen, windowMoveNotify);
-    (*w->screen->windowMoveNotify) (w, dx, dy, immediate);
-    WRAP (ss, w->screen, windowMoveNotify, scaleWindowMoveNotify);
-}
-
 static void
 scalePreparePaintScreen (CompScreen *s,
 			 int	     msSinceLastPaint)
@@ -928,7 +1004,7 @@ scalePreparePaintScreen (CompScreen *s,
     if (ss->state != SCALE_STATE_NONE && ss->state != SCALE_STATE_WAIT)
     {
 	CompWindow *w;
-	int        steps, dx, dy;
+	int        steps;
 	float      amount, chunk;
 
 	amount = msSinceLastPaint * 0.05f * ss->speed;
@@ -953,18 +1029,6 @@ scalePreparePaintScreen (CompScreen *s,
 		    sw->tx += sw->xVelocity * chunk;
 		    sw->ty += sw->yVelocity * chunk;
 		    sw->scale += sw->scaleVelocity * chunk;
-
-		    dx = (sw->restoreX + sw->tx) - w->attrib.x;
-		    dy = (sw->restoreY + sw->ty) - w->attrib.y;
-
-		    ss->scaleMove = TRUE;
-		    moveWindow (w, dx, dy, FALSE, FALSE);
-		    ss->scaleMove = FALSE;
-
-		    (*s->setWindowScale) (w, sw->scale, sw->scale);
-
-		    if (!sw->adjust && ss->state == SCALE_STATE_IN)
-			syncWindowPosition (w);
 		}
 	    }
 
@@ -1022,6 +1086,11 @@ scaleCheckForWindowAt (CompScreen *s,
 	    x2 = w->attrib.x + (w->width  + w->input.right)  * sw->scale;
 	    y2 = w->attrib.y + (w->height + w->input.bottom) * sw->scale;
 
+	    x1 += sw->tx;
+	    y1 += sw->ty;
+	    x2 += sw->tx;
+	    y2 += sw->ty;
+
 	    if (x1 <= x && y1 <= y && x2 > x && y2 > y)
 		return w;
 	}
@@ -1057,6 +1126,30 @@ sendViewportMoveRequest (CompScreen *s,
 		&xev);
 }
 
+static void
+sendDndStatusMessage (CompScreen *s,
+		      Window	 source)
+{
+    XEvent xev;
+
+    SCALE_SCREEN (s);
+
+    xev.xclient.type    = ClientMessage;
+    xev.xclient.display = s->display->display;
+    xev.xclient.format  = 32;
+
+    xev.xclient.message_type = s->display->xdndStatusAtom;
+    xev.xclient.window	     = source;
+
+    xev.xclient.data.l[0] = ss->dndTarget;
+    xev.xclient.data.l[1] = 2;
+    xev.xclient.data.l[2] = 0;
+    xev.xclient.data.l[3] = 0;
+    xev.xclient.data.l[4] = None;
+
+    XSendEvent (s->display->display, source, FALSE, 0, &xev);
+}
+
 static Bool
 scaleTerminate (CompDisplay     *d,
 		CompAction      *action,
@@ -1078,10 +1171,18 @@ scaleTerminate (CompDisplay     *d,
 	if (xid && s->root != xid)
 	    continue;
 
-	if (ss->grabIndex)
+	if (ss->grab)
 	{
-	    removeScreenGrab (s, ss->grabIndex, 0);
-	    ss->grabIndex = 0;
+	    if (ss->grabIndex)
+	    {
+		removeScreenGrab (s, ss->grabIndex, 0);
+		ss->grabIndex = 0;
+	    }
+
+	    if (ss->dndTarget)
+		XUnmapWindow (d->display, ss->dndTarget);
+
+	    ss->grab = FALSE;
 
 	    if (ss->state != SCALE_STATE_NONE)
 	    {
@@ -1103,20 +1204,12 @@ scaleTerminate (CompDisplay     *d,
 		    w = findWindowAtScreen (s, sd->lastActiveWindow);
 		    if (w)
 		    {
-			int saveX = w->attrib.x;
-			int saveY = w->attrib.y;
 			int x, y;
 
-			SCALE_WINDOW (w);
-
-			w->attrib.x = sw->restoreX;
-			w->attrib.y = sw->restoreY;
+			activateWindow (w);
 
 			defaultViewportForWindow (w, &x, &y);
 
-			w->attrib.x = saveX;
-			w->attrib.y = saveY;
-
 			if (x != s->x || y != s->y)
 			    sendViewportMoveRequest (s,
 						     x * s->width,
@@ -1136,6 +1229,39 @@ scaleTerminate (CompDisplay     *d,
     return FALSE;
 }
 
+static Bool
+scaleEnsureDndRedirectWindow (CompScreen *s)
+{
+    SCALE_SCREEN (s);
+
+    if (!ss->dndTarget)
+    {
+	XSetWindowAttributes attr;
+	long		     xdndVersion = 3;
+
+	attr.override_redirect = TRUE;
+
+	ss->dndTarget = XCreateWindow (s->display->display,
+				       s->root,
+				       0, 0, 1, 1, 0,
+				       CopyFromParent,
+				       InputOnly,
+				       CopyFromParent,
+				       CWOverrideRedirect, &attr);
+
+	XChangeProperty (s->display->display, ss->dndTarget,
+			 s->display->xdndAwareAtom,
+			 XA_ATOM, 32, PropModeReplace,
+			 (unsigned char *) &xdndVersion, 1);
+    }
+
+    XMoveResizeWindow (s->display->display, ss->dndTarget,
+		       0, 0, s->width, s->height);
+    XMapRaised (s->display->display, ss->dndTarget);
+
+    return TRUE;
+}
+
 static Bool
 scaleInitiateCommon (CompScreen      *s,
 		     CompAction      *action,
@@ -1146,18 +1272,25 @@ scaleInitiateCommon (CompScreen      *s,
     SCALE_DISPLAY (s->display);
     SCALE_SCREEN (s);
 
+    if (otherScreenGrabExist (s, "scale", 0))
+	return FALSE;
+
     if (!layoutThumbs (s))
 	return FALSE;
 
-    if (!ss->grabIndex)
+    if (state & CompActionStateInitEdgeDnd)
+    {
+	if (scaleEnsureDndRedirectWindow (s))
+	    ss->grab = TRUE;
+    }
+    else if (!ss->grabIndex)
     {
-	if (otherScreenGrabExist (s, "scale", 0))
-	    return FALSE;
-
 	ss->grabIndex = pushScreenGrab (s, ss->cursor, "scale");
+	if (ss->grabIndex)
+	    ss->grab = TRUE;
     }
 
-    if (ss->grabIndex)
+    if (ss->grab)
     {
 	if (!sd->lastActiveNum)
 	    sd->lastActiveNum = s->activeNum - 1;
@@ -1197,7 +1330,7 @@ scaleInitiate (CompDisplay     *d,
 
 	if (ss->state != SCALE_STATE_WAIT && ss->state != SCALE_STATE_OUT)
 	{
-	    ss->allWindows = FALSE;
+	    ss->type = ScaleTypeNormal;
 	    return scaleInitiateCommon (s, action, state, option, nOption);
 	}
     }
@@ -1224,7 +1357,70 @@ scaleInitiateAll (CompDisplay     *d,
 
 	if (ss->state != SCALE_STATE_WAIT && ss->state != SCALE_STATE_OUT)
 	{
-	    ss->allWindows = TRUE;
+	    ss->type = ScaleTypeAll;
+	    return scaleInitiateCommon (s, action, state, option, nOption);
+	}
+    }
+
+    return FALSE;
+}
+
+static Bool
+scaleInitiateGroup (CompDisplay     *d,
+		    CompAction      *action,
+		    CompActionState state,
+		    CompOption      *option,
+		    int		    nOption)
+{
+    CompScreen *s;
+    Window     xid;
+
+    xid = getIntOptionNamed (option, nOption, "root", 0);
+
+    s = findScreenAtDisplay (d, xid);
+    if (s)
+    {
+	SCALE_SCREEN (s);
+
+	if (ss->state != SCALE_STATE_WAIT && ss->state != SCALE_STATE_OUT)
+	{
+	    CompWindow *w;
+
+	    w = findWindowAtDisplay (d, getIntOptionNamed (option, nOption,
+							   "window", 0));
+	    if (w)
+	    {
+		ss->type	 = ScaleTypeGroup;
+		ss->clientLeader = (w->clientLeader) ? w->clientLeader : w->id;
+
+		return scaleInitiateCommon (s, action, state, option, nOption);
+	    }
+	}
+    }
+
+    return FALSE;
+}
+
+static Bool
+scaleInitiateOutput (CompDisplay     *d,
+		     CompAction      *action,
+		     CompActionState state,
+		     CompOption      *option,
+		     int	     nOption)
+{
+    CompScreen *s;
+    Window     xid;
+
+    xid = getIntOptionNamed (option, nOption, "root", 0);
+
+    s = findScreenAtDisplay (d, xid);
+    if (s)
+    {
+	SCALE_SCREEN (s);
+
+	if (ss->state != SCALE_STATE_WAIT && ss->state != SCALE_STATE_OUT)
+	{
+	    ss->type = ScaleTypeOutput;
 	    return scaleInitiateCommon (s, action, state, option, nOption);
 	}
     }
@@ -1248,7 +1444,7 @@ scaleSelectWindowAt (CompScreen *s,
 	sd->lastActiveNum    = w->activeNum;
 	sd->lastActiveWindow = w->id;
 
-	activateWindow (w);
+	moveInputFocusToWindow (w);
 
 	return TRUE;
     }
@@ -1310,7 +1506,7 @@ scaleMoveFocusWindow (CompScreen *s,
 	    sd->lastActiveNum    = focus->activeNum;
 	    sd->lastActiveWindow = focus->id;
 
-	    activateWindow (focus);
+	    moveInputFocusToWindow (focus);
 	}
     }
 }
@@ -1346,6 +1542,32 @@ scaleWindowRemove (CompDisplay *d,
     }
 }
 
+static Bool
+scaleHoverTimeout (void *closure)
+{
+    CompScreen *s = closure;
+
+    SCALE_SCREEN (s);
+    SCALE_DISPLAY (s->display);
+
+    if (ss->grab && ss->state != SCALE_STATE_IN)
+    {
+	CompOption o;
+	CompAction *action =
+	    &sd->opt[SCALE_DISPLAY_OPTION_INITIATE].value.action;
+
+	o.type    = CompOptionTypeInt;
+	o.name    = "root";
+	o.value.i = s->root;
+
+	scaleTerminate (s->display, action, 0, &o, 1);
+    }
+
+    ss->hoverHandle = 0;
+
+    return FALSE;
+}
+
 static void
 scaleHandleEvent (CompDisplay *d,
 		  XEvent      *event)
@@ -1426,6 +1648,91 @@ scaleHandleEvent (CompDisplay *d,
 				     event->xmotion.x_root,
 				     event->xmotion.y_root);
 	}
+	break;
+    case ClientMessage:
+	if (event->xclient.message_type == d->xdndPositionAtom)
+	{
+	    CompWindow *w;
+
+	    w = findWindowAtDisplay (d, event->xclient.window);
+	    if (w)
+	    {
+		SCALE_SCREEN (w->screen);
+
+		s = w->screen;
+
+		if (w->id == ss->dndTarget)
+		    sendDndStatusMessage (w->screen, event->xclient.data.l[0]);
+
+		if (ss->grab			&&
+		    ss->state != SCALE_STATE_IN &&
+		    w->id == ss->dndTarget	&&
+		    ss->opt[SCALE_SCREEN_OPTION_SLOPPY_FOCUS].value.b)
+		{
+		    int x = event->xclient.data.l[2] >> 16;
+		    int y = event->xclient.data.l[2] & 0xffff;
+
+		    w = scaleCheckForWindowAt (s, x, y);
+		    if (w && isScaleWin (w))
+		    {
+			int time;
+
+			time = ss->opt[SCALE_SCREEN_OPTION_HOVER_TIME].value.i;
+
+			if (ss->hoverHandle)
+			{
+			    if (w->activeNum != sd->lastActiveNum)
+			    {
+				compRemoveTimeout (ss->hoverHandle);
+				ss->hoverHandle = 0;
+			    }
+			}
+
+			if (!ss->hoverHandle)
+			    ss->hoverHandle =
+				compAddTimeout (time,
+						scaleHoverTimeout,
+						s);
+
+			scaleSelectWindowAt (s, x, y);
+		    }
+		    else
+		    {
+			if (ss->hoverHandle)
+			    compRemoveTimeout (ss->hoverHandle);
+
+			ss->hoverHandle = 0;
+		    }
+		}
+	    }
+	}
+	else if (event->xclient.message_type == d->xdndDropAtom ||
+		 event->xclient.message_type == d->xdndLeaveAtom)
+	{
+	    CompWindow *w;
+
+	    w = findWindowAtDisplay (d, event->xclient.window);
+	    if (w)
+	    {
+		CompAction *action =
+		    &sd->opt[SCALE_DISPLAY_OPTION_INITIATE].value.action;
+
+		SCALE_SCREEN (w->screen);
+
+		if (ss->grab			&&
+		    ss->state != SCALE_STATE_IN &&
+		    w->id == ss->dndTarget)
+		{
+		    CompOption o;
+
+		    o.type    = CompOptionTypeInt;
+		    o.name    = "root";
+		    o.value.i = w->screen->root;
+
+		    scaleTerminate (d, action, 0, &o, 1);
+		}
+	    }
+	}
     default:
 	break;
     }
@@ -1450,13 +1757,13 @@ scaleDamageWindowRect (CompWindow *w,
 		       Bool	  initial,
 		       BoxPtr     rect)
 {
-    Bool status;
+    Bool status = FALSE;
 
     SCALE_SCREEN (w->screen);
 
     if (initial)
     {
-	if (ss->grabIndex && isScaleWin (w))
+	if (ss->grab && isScaleWin (w))
 	{
 	    if (layoutThumbs (w->screen))
 	    {
@@ -1465,9 +1772,25 @@ scaleDamageWindowRect (CompWindow *w,
 	    }
 	}
     }
+    else if (ss->state == SCALE_STATE_WAIT)
+    {
+	SCALE_WINDOW (w);
+
+	if (sw->slot)
+	{
+	    damageTransformedWindowRect (w,
+					 sw->scale,
+					 sw->scale,
+					 sw->tx,
+					 sw->ty,
+					 rect);
+
+	    status = TRUE;
+	}
+    }
 
     UNWRAP (ss, w->screen, damageWindowRect);
-    status = (*w->screen->damageWindowRect) (w, initial, rect);
+    status |= (*w->screen->damageWindowRect) (w, initial, rect);
     WRAP (ss, w->screen, damageWindowRect, scaleDamageWindowRect);
 
     return status;
@@ -1501,6 +1824,8 @@ scaleSetDisplayOption (CompDisplay     *display,
     switch (index) {
     case SCALE_DISPLAY_OPTION_INITIATE:
     case SCALE_DISPLAY_OPTION_INITIATE_ALL:
+    case SCALE_DISPLAY_OPTION_INITIATE_GROUP:
+    case SCALE_DISPLAY_OPTION_INITIATE_OUTPUT:
 	if (setDisplayAction (display, o, value))
 	    return TRUE;
     default:
@@ -1526,8 +1851,10 @@ scaleDisplayInitOptions (ScaleDisplay *sd,
     o->value.action.bell	  = FALSE;
     o->value.action.edgeMask	  = (1 << SCREEN_EDGE_TOPRIGHT);
     o->value.action.state	  = CompActionStateInitEdge;
+    o->value.action.state	 |= CompActionStateInitEdgeDnd;
     o->value.action.type	  = CompBindingTypeKey;
     o->value.action.state	 |= CompActionStateInitKey;
+    o->value.action.state	 |= CompActionStateInitButton;
     o->value.action.key.modifiers = SCALE_INITIATE_MODIFIERS_DEFAULT;
     o->value.action.key.keycode   =
 	XKeysymToKeycode (display,
@@ -1543,8 +1870,43 @@ scaleDisplayInitOptions (ScaleDisplay *sd,
     o->value.action.bell      = FALSE;
     o->value.action.edgeMask  = 0;
     o->value.action.state     = CompActionStateInitEdge;
+    o->value.action.state    |= CompActionStateInitEdgeDnd;
     o->value.action.type      = 0;
     o->value.action.state    |= CompActionStateInitKey;
+    o->value.action.state    |= CompActionStateInitButton;
+
+    o = &sd->opt[SCALE_DISPLAY_OPTION_INITIATE_GROUP];
+    o->name		      = "initiate_group";
+    o->shortDesc	      = N_("Initiate Window Picker For Window Group");
+    o->longDesc		      = N_("Layout and start transforming "
+				   "window group");
+    o->type		      = CompOptionTypeAction;
+    o->value.action.initiate  = scaleInitiateGroup;
+    o->value.action.terminate = scaleTerminate;
+    o->value.action.bell      = FALSE;
+    o->value.action.edgeMask  = 0;
+    o->value.action.state     = CompActionStateInitEdge;
+    o->value.action.state    |= CompActionStateInitEdgeDnd;
+    o->value.action.type      = 0;
+    o->value.action.state    |= CompActionStateInitKey;
+    o->value.action.state    |= CompActionStateInitButton;
+
+    o = &sd->opt[SCALE_DISPLAY_OPTION_INITIATE_OUTPUT];
+    o->name		      = "initiate_output";
+    o->shortDesc	      = N_("Initiate Window Picker For Windows on "
+				   "Current Output");
+    o->longDesc		      = N_("Layout and start transforming "
+				   "windows on current output");
+    o->type		      = CompOptionTypeAction;
+    o->value.action.initiate  = scaleInitiateOutput;
+    o->value.action.terminate = scaleTerminate;
+    o->value.action.bell      = FALSE;
+    o->value.action.edgeMask  = 0;
+    o->value.action.state     = CompActionStateInitEdge;
+    o->value.action.state    |= CompActionStateInitEdgeDnd;
+    o->value.action.type      = 0;
+    o->value.action.state    |= CompActionStateInitKey;
+    o->value.action.state    |= CompActionStateInitButton;
 }
 
 static Bool
@@ -1612,8 +1974,13 @@ scaleInitScreen (CompPlugin *p,
 	return FALSE;
     }
 
+    ss->grab      = FALSE;
     ss->grabIndex = 0;
 
+    ss->hoverHandle = 0;
+
+    ss->dndTarget = None;
+
     ss->state = SCALE_STATE_NONE;
 
     ss->slots = 0;
@@ -1632,20 +1999,21 @@ scaleInitScreen (CompPlugin *p,
 
     ss->iconOverlay = ScaleIconEmblem;
 
-    ss->scaleMove = FALSE;
-
     scaleScreenInitOptions (ss);
 
     addScreenAction (s, &sd->opt[SCALE_DISPLAY_OPTION_INITIATE].value.action);
     addScreenAction (s,
 		     &sd->opt[SCALE_DISPLAY_OPTION_INITIATE_ALL].value.action);
+    addScreenAction (s,
+		     &sd->opt[SCALE_DISPLAY_OPTION_INITIATE_GROUP].value.action);
+    addScreenAction (s,
+		     &sd->opt[SCALE_DISPLAY_OPTION_INITIATE_OUTPUT].value.action);
 
     WRAP (ss, s, preparePaintScreen, scalePreparePaintScreen);
     WRAP (ss, s, donePaintScreen, scaleDonePaintScreen);
     WRAP (ss, s, paintScreen, scalePaintScreen);
     WRAP (ss, s, paintWindow, scalePaintWindow);
     WRAP (ss, s, damageWindowRect, scaleDamageWindowRect);
-    WRAP (ss, s, windowMoveNotify, scaleWindowMoveNotify);
 
     ss->cursor = XCreateFontCursor (s->display->display, XC_left_ptr);
 
@@ -1665,7 +2033,6 @@ scaleFiniScreen (CompPlugin *p,
     UNWRAP (ss, s, paintScreen);
     UNWRAP (ss, s, paintWindow);
     UNWRAP (ss, s, damageWindowRect);
-    UNWRAP (ss, s, windowMoveNotify);
 
     if (ss->slotsSize)
 	free (ss->slots);
@@ -1695,8 +2062,7 @@ scaleInitWindow (CompPlugin *p,
     sw->xVelocity = sw->yVelocity = 0.0f;
     sw->scaleVelocity = 1.0f;
     sw->delta = 1.0f;
-    sw->restoreX = 0;
-    sw->restoreY = 0;
+    sw->lastThumbOpacity = 0.0f;
 
     w->privates[ss->windowPrivateIndex].ptr = sw;
 
